@@ -95,10 +95,10 @@
         ("Screens" (make-screens)))
       ("Tiny" (make-switch 'tiny-switch)))
   (-> "Unroll"
-      ("Default" (make-switch 'unroll))
-      ("Compressed" (make-switch 'unroll-compressed)) 
-      ("Phantoms" (make-switch 'unroll-phantoms))
-      ("Greyed" (make-switch 'unroll-greyed))
+      ("Default" (make-unroll 'unroll))
+      ("Compressed" (make-unroll 'unroll-compressed))
+      ("Phantoms" (make-unroll 'unroll-phantoms))
+      ("Greyed" (make-unroll 'unroll-greyed))
       ---
       ("Itemize" (make-switch-list 'unroll 'itemize))
       ("Enumerate" (make-switch-list 'unroll 'enumerate))
@@ -252,11 +252,25 @@
         (begin
           (tree-assign! t `(document ,(tree-ref t 0)
                                      (screens (shown (document ,@(cdr l))))))
-          (if (!= (car p) 0)
+          (if (and p (!= (car p) 0))
               (apply tree-go-to `(,t 1 0 0 ,(- (car p) 1) ,@(cdr p)))))
         (begin
           (tree-assign! t `(document (screens (shown (document ,@l)))))
-          (apply tree-go-to `(,t 0 0 0 ,@p))))))
+          (if p (apply tree-go-to `(,t 0 0 0 ,@p)))))))
+
+(tm-define (remove-single-screens)
+  (let* ((t (buffer-tree))
+         (l (tree-children t)))
+    (when (tree-func? (cAr l) 'screens 1)
+      (when (tree-func? (tree-ref (cAr l) 0) 'shown 1)
+        (let* ((d (tree-ref (tree-ref (cAr l) 0) 0))
+               (b (tree->path t))
+               (p (cursor-inside? d)))
+          (when (tree-func? d 'document)
+            (with q (and (pair? p)
+                         (cons (+ (car p) (length (cDr l))) (cdr p)))
+              (tree-assign! t `(document ,@(cDr l) ,@(tree-children d)))
+              (if q (delayed (:idle 1) (go-to-path (append b q)))))))))))
 
 (tm-define (document-propose-screens?)
   (and (style-has? "beamer-style")
@@ -264,7 +278,7 @@
 
 (tm-menu (focus-document-extra-menu t)
   (:require (document-propose-screens?))
-  ("Screens" (make-switch 'screens)))
+  ("Screens" (make-screens)))
 
 (tm-menu (focus-document-extra-icons t)
   (:require (document-propose-screens?))
@@ -272,6 +286,12 @@
     ((balloon "Screens" "Make a multi-slide presentation")
      (make-screens))))
 
+(tm-define (notify-new-style style)
+  (former style)
+  (if (style-has? "beamer-style")
+      (if (not (screens-buffer?)) (make-screens))
+      (if (screens-buffer?) (remove-single-screens))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Slide titles
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -418,6 +438,12 @@
   (=> (balloon (eval (upcase-first (get-init "page-type")))
                "Paper size")
       (link document-page-size-menu))
+  (=> (balloon (eval (upcase-first (font-family-main (get-init "font"))))
+               "Main document font")
+      (link document-short-font-menu))
+  (=> (balloon (eval (string-append (get-init "font-base-size") "pt"))
+               "Font size")
+      (link document-font-base-size-menu))
   (=> (balloon (icon (eval (current-page-icon))) "Page layout")
       (link page-rendering-menu))
   (assuming (focus-has-preferences? t)
